@@ -1,5 +1,5 @@
 /* server.c --- DIGEST-MD5 mechanism from RFC 2831, server side.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010  Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -42,6 +42,7 @@
 #include "session.h"
 #include "digesthmac.h"
 #include "validate.h"
+#include "qop.h"
 
 #define NONCE_ENTROPY_BYTES 16
 
@@ -83,7 +84,7 @@ _gsasl_digest_md5_server_start (Gsasl_session * sctx, void **mech_data)
       return GSASL_MALLOC_ERROR;
     }
 
-  state->challenge.qops = DIGEST_MD5_QOP_AUTH | DIGEST_MD5_QOP_AUTH_INT;
+  state->challenge.qops = DIGEST_MD5_QOP_AUTH;
   state->challenge.ciphers = 0;
 
   state->challenge.nonce = p;
@@ -173,7 +174,27 @@ _gsasl_digest_md5_server_step (Gsasl_session * sctx,
 	  }
       }
 
-      /* FIXME: qop, cipher, maxbuf, more realms. */
+      /* Set QOP */
+      {
+	const char *qopstr = gsasl_property_get (sctx, GSASL_QOPS);
+
+	if (qopstr)
+	  {
+	    int qops = digest_md5_qopstr2qops (qopstr);
+
+	    if (qops == -1)
+	      return GSASL_MALLOC_ERROR;
+
+	    /* We don't support confidentiality right now. */
+	    if (qops & DIGEST_MD5_QOP_AUTH_CONF)
+	      return GSASL_AUTHENTICATION_ERROR;
+
+	    if (qops)
+	      state->challenge.qops = qops;
+	  }
+      }
+
+      /* FIXME: cipher, maxbuf, more realms. */
 
       /* Create challenge. */
       *output = digest_md5_print_challenge (&state->challenge);
@@ -219,7 +240,7 @@ _gsasl_digest_md5_server_step (Gsasl_session * sctx,
 	}
       gsasl_property_set (sctx, GSASL_AUTHZID, state->response.authzid);
 
-      /* FIXME: qop, cipher, maxbuf.  */
+      /* FIXME: cipher, maxbuf.  */
 
       /* Compute secret. */
       {
@@ -274,7 +295,8 @@ _gsasl_digest_md5_server_step (Gsasl_session * sctx,
 			      state->response.cnonce, state->response.qop,
 			      state->response.authzid,
 			      state->response.digesturi, 0,
-			      state->response.cipher, NULL, NULL, NULL, NULL);
+			      state->response.cipher,
+			      state->kic, state->kis, state->kcc, state->kcs);
 	if (rc)
 	  return GSASL_AUTHENTICATION_ERROR;
 
@@ -298,12 +320,6 @@ _gsasl_digest_md5_server_step (Gsasl_session * sctx,
 
       *output_len = strlen (*output);
 
-      state->step++;
-      res = GSASL_NEEDS_MORE;
-      break;
-
-    case 2:
-      *output_len = 0;
       state->step++;
       res = GSASL_OK;
       break;
