@@ -1,5 +1,5 @@
 /* server.c --- SASL mechanism GS2, server side.
- * Copyright (C) 2002, 2003, 2004, 2005, 2006  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2010  Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -33,20 +33,7 @@
 /* Get memcpy, strlen. */
 #include <string.h>
 
-#ifdef HAVE_LIBGSS
-# include <gss.h>
-#elif HAVE_GSSAPI_H		/* Heimdal GSSAPI */
-# include <gssapi.h>
-#else /* MIT GSSAPI */
-# ifdef HAVE_GSSAPI_GSSAPI_H
-#  include <gssapi/gssapi.h>
-# endif
-# ifdef HAVE_GSSAPI_GSSAPI_GENERIC_H
-#  include <gssapi/gssapi_generic.h>
-# endif
-#endif
-
-#include "gs2parser.h"
+#include "gs2helper.h"
 
 struct _Gsasl_gs2_server_state
 {
@@ -54,18 +41,23 @@ struct _Gsasl_gs2_server_state
   gss_name_t client;
   gss_cred_id_t cred;
   gss_ctx_id_t context;
+  gss_OID mech_oid;
+  struct gss_channel_bindings_struct cb;
 };
 typedef struct _Gsasl_gs2_server_state _Gsasl_gs2_server_state;
 
-int
-_gsasl_gs2_server_start (Gsasl_session * sctx, void **mech_data)
+/* Populate state->cred with credential to use for connection. */
+static int
+gs2_get_cred  (Gsasl_session * sctx, _Gsasl_gs2_server_state * state)
 {
-  _Gsasl_gs2_server_state *state;
   OM_uint32 maj_stat, min_stat;
-  gss_name_t server;
   gss_buffer_desc bufdesc;
   const char *service;
   const char *hostname;
+  gss_name_t server;
+  gss_OID_set_desc oid_set;
+  gss_OID_set actual_mechs;
+  int present;
 
   service = gsasl_property_get (sctx, GSASL_SERVICE);
   if (!service)
@@ -75,50 +67,152 @@ _gsasl_gs2_server_start (Gsasl_session * sctx, void **mech_data)
   if (!hostname)
     return GSASL_NO_HOSTNAME;
 
-  /* FIXME: Use asprintf. */
-
-  bufdesc.length = strlen (service) + strlen ("@") + strlen (hostname) + 1;
-  bufdesc.value = malloc (bufdesc.length);
-  if (bufdesc.value == NULL)
+  bufdesc.length = asprintf ((char**) &bufdesc.value, "%s@%s",
+			     service, hostname);
+  if (bufdesc.length <= 0 || bufdesc.value == NULL)
     return GSASL_MALLOC_ERROR;
 
-  sprintf (bufdesc.value, "%s@%s", service, hostname);
-
-  state = (_Gsasl_gs2_server_state *) malloc (sizeof (*state));
-  if (state == NULL)
-    {
-      free (bufdesc.value);
-      return GSASL_MALLOC_ERROR;
-    }
-
-  maj_stat = gss_import_name (&min_stat, &bufdesc, GSS_C_NT_HOSTBASED_SERVICE,
+  maj_stat = gss_import_name (&min_stat, &bufdesc,
+			      GSS_C_NT_HOSTBASED_SERVICE,
 			      &server);
   free (bufdesc.value);
   if (GSS_ERROR (maj_stat))
-    {
-      free (state);
-      return GSASL_GSSAPI_IMPORT_NAME_ERROR;
-    }
+    return GSASL_GSSAPI_IMPORT_NAME_ERROR;
+
+  oid_set.count = 1;
+  oid_set.elements = state->mech_oid;
 
   maj_stat = gss_acquire_cred (&min_stat, server, 0,
-			       GSS_C_NULL_OID_SET, GSS_C_ACCEPT,
-			       &state->cred, NULL, NULL);
+			       &oid_set, GSS_C_ACCEPT,
+			       &state->cred, &actual_mechs, NULL);
   gss_release_name (&min_stat, &server);
+  if (GSS_ERROR (maj_stat))
+    return GSASL_GSSAPI_ACQUIRE_CRED_ERROR;
 
+  maj_stat = gss_test_oid_set_member (&min_stat, state->mech_oid,
+				      actual_mechs, &present);
   if (GSS_ERROR (maj_stat))
+    {
+      gss_release_oid_set (&min_stat, &actual_mechs);
+      return GSASL_GSSAPI_TEST_OID_SET_MEMBER_ERROR;
+    }
+
+  maj_stat = gss_release_oid_set (&min_stat, &actual_mechs);
+  if (GSS_ERROR (maj_stat))
+    return GSASL_GSSAPI_RELEASE_OID_SET_ERROR;
+
+  return GSASL_OK;
+}
+
+int
+_gsasl_gs2_server_start (Gsasl_session * sctx, void **mech_data)
+{
+  _Gsasl_gs2_server_state *state;
+  int res;
+
+  state = (_Gsasl_gs2_server_state *) malloc (sizeof (*state));
+  if (state == NULL)
+    return GSASL_MALLOC_ERROR;
+
+  res = gs2_get_oid (sctx, &state->mech_oid);
+  if (res != GSASL_OK)
     {
       free (state);
-      return GSASL_GSSAPI_ACQUIRE_CRED_ERROR;
+      return res;
+    }
+
+  res = gs2_get_cred (sctx, state);
+  if (res != GSASL_OK)
+    {
+      free (state);
+      return res;
     }
 
   state->step = 0;
   state->context = GSS_C_NO_CONTEXT;
   state->client = NULL;
+
+  /* The initiator-address-type and acceptor-address-type fields of
+     the GSS-CHANNEL-BINDINGS structure MUST be set to 0.  The
+     initiator-address and acceptor-address fields MUST be the empty
+     string. */
+  state->cb.initiator_addrtype = 0;
+  state->cb.initiator_address.length = 0;
+  state->cb.initiator_address.value = NULL;
+  state->cb.acceptor_addrtype = 0;
+  state->cb.acceptor_address.length = 0;
+  state->cb.acceptor_address.value = NULL;
+  state->cb.application_data.length = 0;
+  state->cb.application_data.value = NULL;
+
   *mech_data = state;
 
   return GSASL_OK;
 }
 
+static char *
+unescape_authzid (const char *str, size_t len)
+{
+  char *out = malloc (len + 1);
+  char *p = out;
+
+  if (!out)
+    return NULL;
+
+  while (len > 0 && *str)
+    {
+      if (len >= 3 && str[0] == '=' && str[1] == '2' && str[2] == 'C')
+	{
+	  *p++ = ',';
+	  str += 3;
+	  len -= 3;
+	}
+      else if (len >= 3 && str[0] == '=' && str[1] == '3' && str[2] == 'D')
+	{
+	  *p++ = '=';
+	  str += 3;
+	  len -= 3;
+	}
+      else
+	{
+	  *p++ = *str;
+	  str++;
+	  len--;
+	}
+    }
+  *p = '\0';
+
+  return out;
+}
+
+static int
+parse_gs2_header (const char *data, size_t len,
+		  char **authzid, size_t *headerlen)
+{
+  char *authzid_endptr;
+
+  if (len < 3)
+    return GSASL_MECHANISM_PARSE_ERROR;
+
+  if (strncmp (data, "n,,", 3) == 0)
+    {
+      *headerlen = 3;
+      *authzid = NULL;
+    }
+  else if (strncmp (data, "n,a=", 4) == 0 &&
+	   (authzid_endptr = memchr (data + 4, ',', len - 4)))
+    {
+      *authzid = unescape_authzid (data + 4, authzid_endptr - (data + 4));
+      if (!*authzid)
+	return GSASL_MALLOC_ERROR;
+      *headerlen = authzid_endptr - data + 1;
+    }
+  else
+    return GSASL_MECHANISM_PARSE_ERROR;
+
+  return GSASL_OK;
+}
+
 int
 _gsasl_gs2_server_step (Gsasl_session * sctx,
 			void *mech_data,
@@ -130,13 +224,14 @@ _gsasl_gs2_server_step (Gsasl_session * sctx,
   OM_uint32 maj_stat, min_stat;
   gss_buffer_desc client_name;
   gss_OID mech_type;
-  char tmp[4];
   int res;
   OM_uint32 ret_flags;
-  struct gs2_token tok;
+  int free_bufdesc1 = 0;
 
   *output = NULL;
   *output_len = 0;
+  bufdesc1.value = input;
+  bufdesc1.length = input_len;
 
   switch (state->step)
     {
@@ -150,12 +245,37 @@ _gsasl_gs2_server_step (Gsasl_session * sctx,
       /* fall through */
 
     case 1:
-      res = gs2_parser (input, input_len, &tok);
-      if (res < 0)
-	return GSASL_MECHANISM_PARSE_ERROR;
+      {
+	char *authzid;
+	size_t headerlen;
 
-      bufdesc1.value = tok.context_token;
-      bufdesc1.length = tok.context_length;
+	res = parse_gs2_header (input, input_len, &authzid, &headerlen);
+	if (res != GSASL_OK)
+	  return res;
+
+	if (authzid)
+	  {
+	    gsasl_property_set (sctx, GSASL_AUTHZID, authzid);
+	    free (authzid);
+	  }
+
+	state->cb.application_data.value = input;
+	state->cb.application_data.length = headerlen;
+
+	bufdesc2.value = input + headerlen;
+	bufdesc2.length = input_len - headerlen;
+
+	maj_stat = gss_encapsulate_token (&bufdesc2, state->mech_oid,
+					  &bufdesc1);
+	if (GSS_ERROR (maj_stat))
+	  return GSASL_GSSAPI_ENCAPSULATE_TOKEN_ERROR;
+
+	free_bufdesc1 = 1;
+      }
+      state->step++;
+      /* fall through */
+
+    case 2:
       if (state->client)
 	{
 	  gss_release_name (&min_stat, &state->client);
@@ -166,45 +286,42 @@ _gsasl_gs2_server_step (Gsasl_session * sctx,
 					 &state->context,
 					 state->cred,
 					 &bufdesc1,
-					 GSS_C_NO_CHANNEL_BINDINGS,
+					 &state->cb,
 					 &state->client,
 					 &mech_type,
-					 &bufdesc2, &ret_flags, NULL, NULL);
+					 &bufdesc2,
+					 &ret_flags,
+					 NULL,
+					 NULL);
       if (maj_stat != GSS_S_COMPLETE && maj_stat != GSS_S_CONTINUE_NEEDED)
 	return GSASL_GSSAPI_ACCEPT_SEC_CONTEXT_ERROR;
 
-      /* XXX check wrap token */
-
-      if (ret_flags & GSS_C_PROT_READY_FLAG)
-	{
-	  puts ("prot_ready");
-	  /* XXX gss_wrap token */
-	}
-
-      res = gs2_encode (bufdesc2.value, bufdesc2.length,
-			NULL, 0, output, output_len);
-      if (res < 0)
-	return GSASL_GSSAPI_INIT_SEC_CONTEXT_ERROR;
-
-      maj_stat = gss_release_buffer (&min_stat, &bufdesc2);
-      if (GSS_ERROR (maj_stat))
-	return GSASL_GSSAPI_RELEASE_BUFFER_ERROR;
-
       if (maj_stat == GSS_S_COMPLETE)
-	state->step++;
+	{
+	  state->step++;
 
-      res = GSASL_NEEDS_MORE;
-      break;
+	  if (!(ret_flags & GSS_C_MUTUAL_FLAG))
+	    return GSASL_MECHANISM_PARSE_ERROR;
 
-    case 2:
-      memset (tmp, 0xFF, 4);
-      tmp[0] = GSASL_QOP_AUTH;
-      bufdesc1.length = 4;
-      bufdesc1.value = tmp;
-      maj_stat = gss_wrap (&min_stat, state->context, 0, GSS_C_QOP_DEFAULT,
-			   &bufdesc1, NULL, &bufdesc2);
-      if (GSS_ERROR (maj_stat))
-	return GSASL_GSSAPI_WRAP_ERROR;
+	  maj_stat = gss_display_name (&min_stat, state->client,
+				       &client_name, &mech_type);
+	  if (GSS_ERROR (maj_stat))
+	    return GSASL_GSSAPI_DISPLAY_NAME_ERROR;
+
+	  gsasl_property_set_raw (sctx, GSASL_GSSAPI_DISPLAY_NAME,
+				  client_name.value, client_name.length);
+
+	  res = gsasl_callback (NULL, sctx, GSASL_VALIDATE_GSSAPI);
+	}
+      else
+	res = GSASL_NEEDS_MORE;
+
+      if (free_bufdesc1)
+	{
+	  maj_stat = gss_release_buffer (&min_stat, &bufdesc1);
+	  if (GSS_ERROR (maj_stat))
+	    return GSASL_GSSAPI_RELEASE_BUFFER_ERROR;
+	}
 
       *output = malloc (bufdesc2.length);
       if (!*output)
@@ -215,60 +332,6 @@ _gsasl_gs2_server_step (Gsasl_session * sctx,
       maj_stat = gss_release_buffer (&min_stat, &bufdesc2);
       if (GSS_ERROR (maj_stat))
 	return GSASL_GSSAPI_RELEASE_BUFFER_ERROR;
-
-      state->step++;
-      res = GSASL_NEEDS_MORE;
-      break;
-
-    case 3:
-      bufdesc1.value = (void *) input;
-      bufdesc1.length = input_len;
-      maj_stat = gss_unwrap (&min_stat, state->context, &bufdesc1,
-			     &bufdesc2, NULL, NULL);
-      if (GSS_ERROR (maj_stat))
-	return GSASL_GSSAPI_UNWRAP_ERROR;
-
-      /* [RFC 2222 section 7.2.1]:
-         The client passes this token to GSS_Unwrap and interprets the
-         first octet of resulting cleartext as a bit-mask specifying
-         the security layers supported by the server and the second
-         through fourth octets as the maximum size output_message to
-         send to the server.  The client then constructs data, with
-         the first octet containing the bit-mask specifying the
-         selected security layer, the second through fourth octets
-         containing in network byte order the maximum size
-         output_message the client is able to receive, and the
-         remaining octets containing the authorization identity.  The
-         client passes the data to GSS_Wrap with conf_flag set to
-         FALSE, and responds with the generated output_message.  The
-         client can then consider the server authenticated. */
-
-      if ((((char *) bufdesc2.value)[0] & GSASL_QOP_AUTH) == 0)
-	{
-	  /* Integrity or privacy unsupported */
-	  maj_stat = gss_release_buffer (&min_stat, &bufdesc2);
-	  return GSASL_GSSAPI_UNSUPPORTED_PROTECTION_ERROR;
-	}
-
-      gsasl_property_set_raw (sctx, GSASL_AUTHZID,
-			      (char *) bufdesc2.value + 4,
-			      bufdesc2.length - 4);
-
-      maj_stat = gss_display_name (&min_stat, state->client,
-				   &client_name, &mech_type);
-      if (GSS_ERROR (maj_stat))
-	return GSASL_GSSAPI_DISPLAY_NAME_ERROR;
-
-      gsasl_property_set_raw (sctx, GSASL_GSSAPI_DISPLAY_NAME,
-			      client_name.value, client_name.length);
-
-      maj_stat = gss_release_buffer (&min_stat, &bufdesc2);
-      if (GSS_ERROR (maj_stat))
-	return GSASL_GSSAPI_RELEASE_BUFFER_ERROR;
-
-      res = gsasl_callback (NULL, sctx, GSASL_VALIDATE_GSSAPI);
-
-      state->step++;
       break;
 
     default:
